using System;
using System.Data.SqlClient;
using System.IO;
using System.Web;
using System.Configuration;
using AspNetGallery.Extensions.Utilities;


namespace AspNetGallery.Extensions.Handlers
{
    public class NorthwindPhotoImageHandler : IHttpHandler
    {
        #region IHttpHandler
        public void ProcessRequest(HttpContext context)
        {
            // Ensure the URL contains an ID argument being a number
            var id = -1; 
            var p1 = context.Request.Params["id"];
            if (p1 != null)
                id = p1.ToInt32(-1);
            if (id < 0)
            {
                context.Response.End();
            }

            // Of course this is ARGUABLY the best thing to do :)
            // You might want to use an external and configurable component (provider)
            // to return an array of bytes or just the command text and query string.
            var connString = ConfigurationManager.ConnectionStrings["NorthwindConnectionString"].ConnectionString;
            const String cmdText = "SELECT photo FROM employees WHERE employeeid=@id";

            // Get an array of bytes from the BLOB field
            byte[] img = null;
            var conn = new SqlConnection(connString);
            using (conn)
            {
                var cmd = new SqlCommand(cmdText, conn);
                cmd.Parameters.AddWithValue("@id", id);
                conn.Open();
                img = (byte[])cmd.ExecuteScalar();
                conn.Close();
            }

            // Prepare the response for the browser
            if (img != null)
            {
                context.Response.ContentType = "image/jpeg";
                //context.Response.BinaryWrite(img);

                // The 78-byte offset is required because of the particular 
                //(and non-standard) structure of the pictures stored in the 
                // Employees table of the Northwind database
                const Int32 magicOffset = 78;
                context.Response.OutputStream.Write(img, magicOffset, img.Length - magicOffset);
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }
        #endregion

    }
}