using System;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using AspNetGallery.Extensions.Utilities;
using System.Collections.Generic;

namespace AspNetGallery.Extensions.Handlers
{
    public class PictureViewerHandler : IHttpHandler
    {
        // Override the ProcessRequest method
        public void ProcessRequest(HttpContext context)
        {
            var info = GetFolderInfo(context);
            var html = CreateOutput(info);

            // Output the data
            context.Response.Write("<html><head><title>");
            context.Response.Write("Picture Web Viewer");
            context.Response.Write("</title></head><body>");
            context.Response.Write(html);
            context.Response.Write("</body></html>");
        }

        // Override the IsReusable property
        public Boolean IsReusable
        {
            get { return true; }
        }


        #region Private Members
        private static PictureViewerInfo GetFolderInfo(HttpContext context)
        {
            // Get optional parameters: Width and Cols
            var info = new PictureViewerInfo();
            var p1 = context.Request.Params["Width"];
            var p2 = context.Request.Params["Cols"];
            if (p1 != null)
                info.DisplayWidth = p1.ToInt32();
            if (p2 != null)
                info.ColumnCount = p2.ToInt32();

            // Gets the portion of the URL that precedes the name of the handler (i.e., "folder.axd")
            var url = context.Request.Url;
            var fileName = url.Segments[url.Segments.Length-1];
            var absolutePath = context.Request.Path;
            fileName = absolutePath.Replace(fileName, "");
            if (fileName.EndsWith("/") || fileName.EndsWith(@"\"))
            {
                fileName = fileName.Substring(0, fileName.Length - 1);
                fileName = fileName.TrimEnd();
            }
            try
            {
                fileName = context.Server.MapPath(fileName);
            }
            catch (Exception e)
            {
                ErrorMessage(context, e.Message);
            }

            info.FolderName = fileName;
            return info;
        }

        private static String CreateOutput(PictureViewerInfo info)
        {
            var pathName = info.FolderName;
            var di = new DirectoryInfo(pathName);
            return di.Exists ? CreateOutputForFolder(info, di) : CreateOutputForSingleImage(pathName);
        }

        private static String CreateOutputForFolder(PictureViewerInfo info, DirectoryInfo di)
        {
            var images = GetAllImages(di);

            var t = new Table();
            var index = 0;
            var moreImages = true;
            while (moreImages)
            {
                var row = new TableRow();
                t.Rows.Add(row);

                for (var i = 0; i < info.ColumnCount; i++)
                {
                    var cell = new TableCell();
                    row.Cells.Add(cell);

                    var img = new Image();
                    var fi = images[index];
                    img.ImageUrl = fi.Name;
                    img.Width = Unit.Pixel(info.DisplayWidth);

                    var a = new HtmlAnchor {HRef = fi.Name};
                    a.Controls.Add(img);

                    cell.Controls.Add(a);

                    index++;
                    moreImages = (index < images.Count);
                    if (!moreImages)
                        break;
                }
            }

            return ControlExtensions.RenderControlAsString(t);
        }

        private static IList<FileInfo> GetAllImages(DirectoryInfo di)
        {
            String[] fileTypes = { "*.bmp", "*.gif", "*.jpg", "*.png" };
            var images = new List<FileInfo>();
            foreach (var files in fileTypes.Select(di.GetFiles).Where(files => files.Length > 0))
            {
                images.AddRange(files);
            }
            return images;
        }

        private static String CreateOutputForSingleImage(String path)
        {
            var img = new Image {ImageUrl = path};
            return  ControlExtensions.RenderControlAsString(img);
        }

        private static void ErrorMessage(HttpContext context, String msg)
        {
            context.Response.Write(msg);
            context.Response.End();
        }
        #endregion
    }
}