﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Xml;

namespace RuntEffects
{
    public partial class AsyncBlog : System.Web.UI.Page
    {
        const String Rssfeed = "http://weblogs.asp.net/despos/rss.aspx";
        private WebRequest _request;

        public String RssData  { get; set;}
        
        protected void Page_Load(object sender, EventArgs e)
        {
            AddOnPreRenderCompleteAsync(BeginTask, EndTask);
        }

        IAsyncResult BeginTask(Object sender,
                           EventArgs e, AsyncCallback cb, Object state)
        {
            // Trace 
            Trace.Warn("Begin async: Thread=" +
                        Thread.CurrentThread.ManagedThreadId.ToString());

            // Prepare to make a Web request for the RSS feed
            _request = WebRequest.Create(Rssfeed);

            // Begin the operation and return an IAsyncResult object  
            return _request.BeginGetResponse(cb, state);
        }

        void EndTask(IAsyncResult ar)
        {
            // This code will be called on a pooled thread 

            using (var response = _request.EndGetResponse(ar))
            {
                String text;
                using (var reader = new StreamReader(response.GetResponseStream()))
                {
                    text = reader.ReadToEnd();
                }

                // Process the RSS data
                RssData = ProcessFeed(text);
            }

            // Trace        
            Trace.Warn("End async: Thread=" +
                        Thread.CurrentThread.ManagedThreadId);

            // The page is updated using an ASP-style code block in the ASPX 
            // source that displays the contents of the rssData variable
        }

        static String ProcessFeed(String feed)
        {
            // Build the page output from the XML input  
            var sb = new StringBuilder();

            var doc = new XmlDocument();
            doc.LoadXml(feed);

            var nav = doc.CreateNavigator();
            if (nav == null)
                return "";

            // Title and description
            var iterator = nav.Select("/rss/channel/title");
            iterator.MoveNext();
            sb.AppendFormat("<h3>{0}</h3>", iterator.Current.Value);
            iterator = nav.Select("/rss/channel/description");
            iterator.MoveNext();
            sb.AppendFormat("<i>{0}</i>", iterator.Current.Value);

            var items = doc.SelectNodes("/rss/channel/item");
            if (items == null)
                return "";

            sb.Append("<ul>");
            foreach (XmlNode n in items)
            {
                var title = n.SelectSingleNode("title");
                var link = n.SelectSingleNode("link");
                var date = n.SelectSingleNode("pubDate");
                sb.AppendFormat("<li><a href=\"{0}\">{1}</a> on {2}</li>", link.InnerText, title.InnerText, date.InnerText);
            }
            sb.Append("</ul>");

            return sb.ToString();
        }
    }
}