using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using HelloSilver.API;

// ref. System.Xml.Linq

namespace HelloSilver.Helpers
{
    public class PackageUtils
    {
        private static XElement GetXmlDocFromDefaultPackage(String fileName)
        {
           var settings = new XmlReaderSettings {XmlResolver = new XmlXapResolver()};
            var reader = XmlReader.Create(fileName, settings); 
            
            // Create an XmlElement from XmlReader 
            var element = XElement.Load(reader);
            return element;
        }

        public static ProductInfo GetInventoryInfo(String id)
        {
            var e = GetXmlDocFromDefaultPackage("Products.xml"); 
            var products = (from product in e.Elements("product") 
                            where (String)product.Attribute("ProductID") == id 
                            select product).ToList();
            if (products.Count == 0)
                return new MissingProductInfo();

            var el = products.ElementAt(0);
            var p = new ProductInfo
                        {
                            ProductId = (Int32) el.Attribute("ProductID"),
                            Name = (String) el.Attribute("ProductName"),
                            ProductNumber = (String) el.Attribute("ProductNumber"),
                            SafetyStockLevel = (Int32) el.Attribute("ProductSafetyStockLevel"),
                            ReorderPoint = (Int32) el.Attribute("ProductReorderPoint")
                        }; 
            return p;
        }

        public static IEnumerable<ProductInfo> GetInventoryInfoByName(String productName) 
        { 
            var productData = GetSampleData();
            var testValue = productName.ToLower();
            var products = (from product in productData
                            where product.Name.ToLower().Contains(testValue) 
                            select product).ToArray(); 
            return products; 
        }

        private static IEnumerable<ProductInfo> GetSampleData() 
        { 
            var products = new List<ProductInfo>
                               {
                                   new ProductInfo
                                       {
                                           ProductId = 1,
                                           Name = "Adjustable Race",
                                           ProductNumber = "AR-5381",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 2,
                                           Name = "Bearing Ball",
                                           ProductNumber = "BA-8327",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 3,
                                           Name = "BB Ball Bearing",
                                           ProductNumber = "BE-2349",
                                           SafetyStockLevel = 800,
                                           ReorderPoint = 600
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 4,
                                           Name = "Headset Ball Bearings",
                                           ProductNumber = "BE-2908",
                                           SafetyStockLevel = 800,
                                           ReorderPoint = 600
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 327,
                                           Name = "Down Tube",
                                           ProductNumber = "DT-2377",
                                           SafetyStockLevel = 800,
                                           ReorderPoint = 600
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 328,
                                           Name = "Mountain End Caps",
                                           ProductNumber = "EC-M092",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 371,
                                           Name = "Thin-Jam Hex Nut 7",
                                           ProductNumber = "HJ-7161",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 372,
                                           Name = "Thin-Jam Hex Nut 8",
                                           ProductNumber = "HJ-7162",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 373,
                                           Name = "Thin-Jam Hex Nut 12",
                                           ProductNumber = "HJ-9080",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       },
                                   new ProductInfo
                                       {
                                           ProductId = 374,
                                           Name = "Thin-Jam Hex Nut 11",
                                           ProductNumber = "HJ-9161",
                                           SafetyStockLevel = 1000,
                                           ReorderPoint = 750
                                       }
                               };
            return products; 
        }
    }
}