using System;
using Presentation.Navigation;
using Presentation.Views.Default;
using ServiceLayer;

namespace Presentation.Presenters.Default
{
    public class DefaultPresenter
    {
        private readonly IDefaultView _view;
        private readonly IQuoteServices _quoteServices;

        public DefaultPresenter(IDefaultView view) : this(view, new QuoteServices())
        {
        }

        public DefaultPresenter(IDefaultView view, IQuoteServices serviceLayer)
        {
            _view = view;
            _quoteServices = serviceLayer;
        }

        public void Refresh()
        {
            // Get input from the view
            var symbols = _view.Symbols;

            // Execute the action
            var stocks = _quoteServices.GetQuotes(symbols);

            // Update the view
            _view.Quotes = stocks;
            _view.Message = String.Format("Data downloaded at: {0}", DateTime.Now);
        }

        public void Redirect()
        {
            Navigator.Goto("test", "test value");
        }
    }
}